import { ActionCreatorWithPayload } from '@reduxjs/toolkit';
import { ElementType, useCallback } from 'react';
import { useAppDispatch } from '../../hooks';
import { extractFormData } from '../../utils';

export const WithHocForm = (Component: ElementType) => {
  return function WithHocFormComponent({ ...props }: {
    [key: string]: any;
  }) {
    const dispatch = useAppDispatch();

    const dispatchFormData = useCallback((
      { currentTarget }: any,
      setFormData: ActionCreatorWithPayload<any>
    ) => {
      const data = extractFormData(currentTarget);

      dispatch(setFormData({
        data: {
          attributes: data,
        },
      }));
    }, [dispatch]);

    return (
      <Component
        handleFormDataChange={(
          e: any,
          setFormData: ActionCreatorWithPayload<any>
        ) => {
          e.preventDefault();
          dispatchFormData(e, setFormData);
          e.target.reset();
        }}
        {...props}
      />
    );
  };
};

export type HandleFormDataChangeType = (
  e: any,
  setFormData: ActionCreatorWithPayload<any>
) => void;
